#!/usr/bin/python
from __future__ import (absolute_import, division, print_function)
# Copyright 2019-2020 Fortinet, Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

__metaclass__ = type

ANSIBLE_METADATA = {'status': ['preview'],
                    'supported_by': 'community',
                    'metadata_version': '1.1'}

DOCUMENTATION = '''
---
module: fortios_wireless_controller_mpsk_profile
short_description: Configure MPSK profile in Fortinet's FortiOS and FortiGate.
description:
    - This module is able to configure a FortiGate or FortiOS (FOS) device by allowing the
      user to set and modify wireless_controller feature and mpsk_profile category.
      Examples include all parameters and values need to be adjusted to datasources before usage.
      Tested with FOS v6.0.0
version_added: "2.10"
author:
    - Link Zheng (@chillancezen)
    - Jie Xue (@JieX19)
    - Hongbin Lu (@fgtdev-hblu)
    - Frank Shen (@frankshen01)
    - Miguel Angel Munoz (@mamunozgonzalez)
    - Nicolas Thomas (@thomnico)
notes:
    - Legacy fortiosapi has been deprecated, httpapi is the preferred way to run playbooks

requirements:
    - ansible>=2.9.0
options:
    access_token:
        description:
            - Token-based authentication.
              Generated from GUI of Fortigate.
        type: str
        required: false
    enable_log:
        description:
            - Enable/Disable logging for task.
        type: bool
        required: false
        default: false
    vdom:
        description:
            - Virtual domain, among those defined previously. A vdom is a
              virtual instance of the FortiGate that can be configured and
              used as a different unit.
        type: str
        default: root

    state:
        description:
            - Indicates whether to create or remove the object.
        type: str
        required: true
        choices:
            - present
            - absent
    wireless_controller_mpsk_profile:
        description:
            - Configure MPSK profile.
        default: null
        type: dict
        suboptions:
            mpsk_concurrent_clients:
                description:
                    - Maximum number of concurrent clients that connect using the same passphrase in multiple PSK authentication (0 - 65535).
                type: int
            mpsk_group:
                description:
                    - List of multiple PSK groups.
                type: list
                suboptions:
                    mpsk_key:
                        description:
                            - List of multiple PSK entries.
                        type: list
                        suboptions:
                            comment:
                                description:
                                    - Comment.
                                type: str
                            concurrent_client_limit_type:
                                description:
                                    - MPSK client limit type options.
                                type: str
                                choices:
                                    - default
                                    - unlimited
                                    - specified
                            concurrent_clients:
                                description:
                                    - Number of clients that can connect using this pre-shared key (1 - 65535).
                                type: int
                            mac:
                                description:
                                    - MAC address.
                                type: str
                            mpsk_schedules:
                                description:
                                    - Firewall schedule for MPSK passphrase. The passphrase will be effective only when at least one schedule is valid.
                                type: list
                                suboptions:
                                    name:
                                        description:
                                            - Schedule name. Source firewall.schedule.group.name firewall.schedule.recurring.name firewall.schedule.onetime
                                              .name.
                                        required: true
                                        type: str
                            name:
                                description:
                                    - Pre-shared key name.
                                required: true
                                type: str
                            passphrase:
                                description:
                                    - WPA Pre-shared key.
                                type: str
                    name:
                        description:
                            - MPSK group name.
                        required: true
                        type: str
                    vlan_id:
                        description:
                            - Optional VLAN ID.
                        type: int
                    vlan_type:
                        description:
                            - MPSK group VLAN options.
                        type: str
                        choices:
                            - no-vlan
                            - fixed-vlan
            name:
                description:
                    - MPSK profile name.
                required: true
                type: str
'''

EXAMPLES = '''
- hosts: fortigates
  collections:
    - fortinet.fortios
  connection: httpapi
  vars:
   vdom: "root"
   ansible_httpapi_use_ssl: yes
   ansible_httpapi_validate_certs: no
   ansible_httpapi_port: 443
  tasks:
  - name: Configure MPSK profile.
    fortios_wireless_controller_mpsk_profile:
      vdom:  "{{ vdom }}"
      state: "present"
      access_token: "<your_own_value>"
      wireless_controller_mpsk_profile:
        mpsk_concurrent_clients: "3"
        mpsk_group:
         -
            mpsk_key:
             -
                comment: "Comment."
                concurrent_client_limit_type: "default"
                concurrent_clients: "8"
                mac: "<your_own_value>"
                mpsk_schedules:
                 -
                    name: "default_name_11 (source firewall.schedule.group.name firewall.schedule.recurring.name firewall.schedule.onetime.name)"
                name: "default_name_12"
                passphrase: "<your_own_value>"
            name: "default_name_14"
            vlan_id: "15"
            vlan_type: "no-vlan"
        name: "default_name_17"

'''

RETURN = '''
build:
  description: Build number of the fortigate image
  returned: always
  type: str
  sample: '1547'
http_method:
  description: Last method used to provision the content into FortiGate
  returned: always
  type: str
  sample: 'PUT'
http_status:
  description: Last result given by FortiGate on last operation applied
  returned: always
  type: str
  sample: "200"
mkey:
  description: Master key (id) used in the last call to FortiGate
  returned: success
  type: str
  sample: "id"
name:
  description: Name of the table used to fulfill the request
  returned: always
  type: str
  sample: "urlfilter"
path:
  description: Path of the table used to fulfill the request
  returned: always
  type: str
  sample: "webfilter"
revision:
  description: Internal revision number
  returned: always
  type: str
  sample: "17.0.2.10658"
serial:
  description: Serial number of the unit
  returned: always
  type: str
  sample: "FGVMEVYYQT3AB5352"
status:
  description: Indication of the operation's result
  returned: always
  type: str
  sample: "success"
vdom:
  description: Virtual domain used
  returned: always
  type: str
  sample: "root"
version:
  description: Version of the FortiGate
  returned: always
  type: str
  sample: "v5.6.3"

'''

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.connection import Connection
from ansible_collections.fortinet.fortios.plugins.module_utils.fortios.fortios import FortiOSHandler
from ansible_collections.fortinet.fortios.plugins.module_utils.fortios.fortios import check_legacy_fortiosapi
from ansible_collections.fortinet.fortios.plugins.module_utils.fortios.fortios import schema_to_module_spec
from ansible_collections.fortinet.fortios.plugins.module_utils.fortios.fortios import check_schema_versioning
from ansible_collections.fortinet.fortios.plugins.module_utils.fortimanager.common import FAIL_SOCKET_MSG


def filter_wireless_controller_mpsk_profile_data(json):
    option_list = ['mpsk_concurrent_clients', 'mpsk_group', 'name']
    dictionary = {}

    for attribute in option_list:
        if attribute in json and json[attribute] is not None:
            dictionary[attribute] = json[attribute]

    return dictionary


def underscore_to_hyphen(data):
    if isinstance(data, list):
        for i, elem in enumerate(data):
            data[i] = underscore_to_hyphen(elem)
    elif isinstance(data, dict):
        new_data = {}
        for k, v in data.items():
            new_data[k.replace('_', '-')] = underscore_to_hyphen(v)
        data = new_data

    return data


def wireless_controller_mpsk_profile(data, fos):
    vdom = data['vdom']

    state = data['state']

    wireless_controller_mpsk_profile_data = data['wireless_controller_mpsk_profile']
    filtered_data = underscore_to_hyphen(filter_wireless_controller_mpsk_profile_data(wireless_controller_mpsk_profile_data))

    if state == "present" or state is True:
        return fos.set('wireless-controller',
                       'mpsk-profile',
                       data=filtered_data,
                       vdom=vdom)

    elif state == "absent":
        return fos.delete('wireless-controller',
                          'mpsk-profile',
                          mkey=filtered_data['name'],
                          vdom=vdom)
    else:
        fos._module.fail_json(msg='state must be present or absent!')


def is_successful_status(status):
    return status['status'] == "success" or \
        status['http_method'] == "DELETE" and status['http_status'] == 404


def fortios_wireless_controller(data, fos):

    if data['wireless_controller_mpsk_profile']:
        resp = wireless_controller_mpsk_profile(data, fos)
    else:
        fos._module.fail_json(msg='missing task body: %s' % ('wireless_controller_mpsk_profile'))

    return not is_successful_status(resp), \
        resp['status'] == "success" and \
        (resp['revision_changed'] if 'revision_changed' in resp else True), \
        resp


versioned_schema = {
    "type": "list",
    "children": {
        "mpsk_concurrent_clients": {
            "type": "integer",
            "revisions": {
                "v6.4.4": True,
                "v6.4.0": True
            }
        },
        "name": {
            "type": "string",
            "revisions": {
                "v6.4.4": True,
                "v6.4.0": True
            }
        },
        "mpsk_group": {
            "type": "list",
            "children": {
                "mpsk_key": {
                    "type": "list",
                    "children": {
                        "comment": {
                            "type": "string",
                            "revisions": {
                                "v6.4.4": True,
                                "v6.4.0": True
                            }
                        },
                        "concurrent_client_limit_type": {
                            "type": "string",
                            "options": [
                                {
                                    "value": "default",
                                    "revisions": {
                                        "v6.4.4": True,
                                        "v6.4.0": True
                                    }
                                },
                                {
                                    "value": "unlimited",
                                    "revisions": {
                                        "v6.4.4": True,
                                        "v6.4.0": True
                                    }
                                },
                                {
                                    "value": "specified",
                                    "revisions": {
                                        "v6.4.4": True,
                                        "v6.4.0": True
                                    }
                                }
                            ],
                            "revisions": {
                                "v6.4.4": True,
                                "v6.4.0": True
                            }
                        },
                        "name": {
                            "type": "string",
                            "revisions": {
                                "v6.4.4": True,
                                "v6.4.0": True
                            }
                        },
                        "mpsk_schedules": {
                            "type": "list",
                            "children": {
                                "name": {
                                    "type": "string",
                                    "revisions": {
                                        "v6.4.4": True,
                                        "v6.4.0": True
                                    }
                                }
                            },
                            "revisions": {
                                "v6.4.4": True,
                                "v6.4.0": True
                            }
                        },
                        "mac": {
                            "type": "string",
                            "revisions": {
                                "v6.4.4": True,
                                "v6.4.0": True
                            }
                        },
                        "passphrase": {
                            "type": "string",
                            "revisions": {
                                "v6.4.4": True,
                                "v6.4.0": True
                            }
                        },
                        "concurrent_clients": {
                            "type": "integer",
                            "revisions": {
                                "v6.4.4": True,
                                "v6.4.0": True
                            }
                        }
                    },
                    "revisions": {
                        "v6.4.4": True,
                        "v6.4.0": True
                    }
                },
                "vlan_type": {
                    "type": "string",
                    "options": [
                        {
                            "value": "no-vlan",
                            "revisions": {
                                "v6.4.4": True,
                                "v6.4.0": True
                            }
                        },
                        {
                            "value": "fixed-vlan",
                            "revisions": {
                                "v6.4.4": True,
                                "v6.4.0": True
                            }
                        }
                    ],
                    "revisions": {
                        "v6.4.4": True,
                        "v6.4.0": True
                    }
                },
                "name": {
                    "type": "string",
                    "revisions": {
                        "v6.4.4": True,
                        "v6.4.0": True
                    }
                },
                "vlan_id": {
                    "type": "integer",
                    "revisions": {
                        "v6.4.4": True,
                        "v6.4.0": True
                    }
                }
            },
            "revisions": {
                "v6.4.4": True,
                "v6.4.0": True
            }
        }
    },
    "revisions": {
        "v6.4.4": True,
        "v6.4.0": True
    }
}


def main():
    module_spec = schema_to_module_spec(versioned_schema)
    mkeyname = 'name'
    fields = {
        "access_token": {"required": False, "type": "str", "no_log": True},
        "enable_log": {"required": False, "type": bool},
        "vdom": {"required": False, "type": "str", "default": "root"},
        "state": {"required": True, "type": "str",
                  "choices": ["present", "absent"]},
        "wireless_controller_mpsk_profile": {
            "required": False, "type": "dict", "default": None,
            "options": {
            }
        }
    }
    for attribute_name in module_spec['options']:
        fields["wireless_controller_mpsk_profile"]['options'][attribute_name] = module_spec['options'][attribute_name]
        if mkeyname and mkeyname == attribute_name:
            fields["wireless_controller_mpsk_profile"]['options'][attribute_name]['required'] = True

    check_legacy_fortiosapi()
    module = AnsibleModule(argument_spec=fields,
                           supports_check_mode=False)

    versions_check_result = None
    if module._socket_path:
        connection = Connection(module._socket_path)
        if 'access_token' in module.params:
            connection.set_option('access_token', module.params['access_token'])

        if 'enable_log' in module.params:
            connection.set_option('enable_log', module.params['enable_log'])
        else:
            connection.set_option('enable_log', False)
        fos = FortiOSHandler(connection, module, mkeyname)
        versions_check_result = check_schema_versioning(fos, versioned_schema, "wireless_controller_mpsk_profile")

        is_error, has_changed, result = fortios_wireless_controller(module.params, fos)

    else:
        module.fail_json(**FAIL_SOCKET_MSG)

    if versions_check_result and versions_check_result['matched'] is False:
        module.warn("Ansible has detected version mismatch between FortOS system and your playbook, see more details by specifying option -vvv")

    if not is_error:
        if versions_check_result and versions_check_result['matched'] is False:
            module.exit_json(changed=has_changed, version_check_warning=versions_check_result, meta=result)
        else:
            module.exit_json(changed=has_changed, meta=result)
    else:
        if versions_check_result and versions_check_result['matched'] is False:
            module.fail_json(msg="Error in repo", version_check_warning=versions_check_result, meta=result)
        else:
            module.fail_json(msg="Error in repo", meta=result)


if __name__ == '__main__':
    main()
